from wagtail.models.i18n import (  # noqa
    BootstrapTranslatableMixin, BootstrapTranslatableModel, Locale, LocaleManager,
    TranslatableMixin, bootstrap_translatable_model, get_translatable_models)
