{% if form.non_field_errors %}
<div class="messages">
    <ul>
        {% for error in form.non_field_errors %}
            <li class="error">{{ error }}</li>
        {% endfor %}
    </ul>
</div>
{% endif %}
