{% load i18n %}
{% load wagtailadmin %}

<nav class="pagination" aria-label="{% trans 'Pagination' %}">
    <p>{% blocktrans with page_num=items.number total_pages=items.paginator.num_pages %}Page {{ page_num }} of {{ total_pages }}.{% endblocktrans %}</p>
    <ul>
        <li class="prev">
            {% if items.has_previous %}
                <a href="#" data-page="{{ items.previous_page_number }}">
                    {% icon name="arrow-left" class_name="default" %}
                    {% trans 'Previous' %}
                </a>
            {% endif %}
        </li>
        <li class="next">
            {% if items.has_next %}
                <a href="#" data-page="{{ items.next_page_number }}">
                    {% trans 'Next' %}
                    {% icon name="arrow-right" class_name="default" %}
                </a>
            {% endif %}
        </li>
    </ul>
</nav>
