(()=>{"use strict";var t,e={83612:(t,e,n)=>{e.__esModule=!0,e.initUpgradeNotification=void 0;var i=n(68722);e.initUpgradeNotification=function(){var t=document.querySelector("[data-upgrade]");if(t){var e="https://releases.wagtail.io/latest.txt",n=t.dataset.wagtailVersion;fetch(e).then((function(t){return 200!==t.status?(console.log("Unexpected response from "+e+". Status: "+t.status),!1):t.json()})).then((function(e){e&&e.version&&i.versionOutOfDate(e.version,n)&&(t.querySelector("[data-upgrade-version]").innerText=e.version,t.querySelector("[data-upgrade-link]").setAttribute("href",e.url),t.style.display="")})).catch((function(t){console.log("Error fetching "+e+". Error: "+t)}))}}},40748:function(t,e,n){var i=this&&this.__importDefault||function(t){return t&&t.__esModule?t:{default:t}};e.__esModule=!0,i(n(14547)),i(n(3130));var o=n(48188);window.wagtail.components={Icon:o.Icon,Portal:o.Portal},document.addEventListener("DOMContentLoaded",(function(){o.initFocusOutline(),o.initIE11Warning(),o.initUpgradeNotification()}))},45733:(t,e,n)=>{e.__esModule=!0,e.initIE11Warning=void 0;var i=n(20434);e.initIE11Warning=function(){i.IS_IE11&&[].slice.call(document.querySelectorAll("[data-ie11-warning]")).forEach((function(t){t.hidden=!1}))}},48188:function(t,e,n){var i=this&&this.__importDefault||function(t){return t&&t.__esModule?t:{default:t}};e.__esModule=!0,e.initUpgradeNotification=e.initIE11Warning=e.initFocusOutline=e.Transition=e.PublicationStatus=e.Portal=e.LoadingSpinner=e.Icon=e.Button=void 0;var o=i(n(47240));e.Button=o.default;var r=i(n(91302));e.Icon=r.default;var a=i(n(51908));e.LoadingSpinner=a.default;var u=i(n(16297));e.Portal=u.default;var l=i(n(19154));e.PublicationStatus=l.default;var s=i(n(99912));e.Transition=s.default;var d=n(10224);e.initFocusOutline=d.initFocusOutline;var c=n(45733);e.initIE11Warning=c.initIE11Warning;var f=n(83612);e.initUpgradeNotification=f.initUpgradeNotification},10224:(t,e)=>{e.__esModule=!0,e.initFocusOutline=void 0;var n=function(t){document.body.classList.toggle("focus-outline-off",!t),document.body.classList.toggle("focus-outline-on",t)},i=n.bind(null,!1),o=n.bind(null,!0);e.initFocusOutline=function(){i(),window.addEventListener("mousedown",i),window.addEventListener("touchstart",i),window.addEventListener("keydown",(function(t){9===t.keyCode&&o()}))}},68722:(t,e)=>{function n(t,e){for(var n=/(\.0)+[^.]*$/,i=(t+"").replace(n,"").split("."),o=(e+"").replace(n,"").split("."),r=Math.min(i.length,o.length),a=0;a<r;a++){var u=parseInt(i[a],10)-parseInt(o[a],10);if(0!==u)return u}return i.length-o.length}e.__esModule=!0,e.versionOutOfDate=e.compareVersion=void 0,e.compareVersion=n,e.versionOutOfDate=function(t,e){return n(t,e)>0}}},n={};function i(t){var o=n[t];if(void 0!==o)return o.exports;var r=n[t]={id:t,loaded:!1,exports:{}};return e[t].call(r.exports,r,r.exports,i),r.loaded=!0,r.exports}i.m=e,t=[],i.O=(e,n,o,r)=>{if(!n){var a=1/0;for(s=0;s<t.length;s++){for(var[n,o,r]=t[s],u=!0,l=0;l<n.length;l++)(!1&r||a>=r)&&Object.keys(i.O).every((t=>i.O[t](n[l])))?n.splice(l--,1):(u=!1,r<a&&(a=r));u&&(t.splice(s--,1),e=o())}return e}r=r||0;for(var s=t.length;s>0&&t[s-1][2]>r;s--)t[s]=t[s-1];t[s]=[n,o,r]},i.n=t=>{var e=t&&t.__esModule?()=>t.default:()=>t;return i.d(e,{a:e}),e},i.d=(t,e)=>{for(var n in e)i.o(e,n)&&!i.o(t,n)&&Object.defineProperty(t,n,{enumerable:!0,get:e[n]})},i.g=function(){if("object"==typeof globalThis)return globalThis;try{return this||new Function("return this")()}catch(t){if("object"==typeof window)return window}}(),i.hmd=t=>((t=Object.create(t)).children||(t.children=[]),Object.defineProperty(t,"exports",{enumerable:!0,set:()=>{throw new Error("ES Modules may not assign module.exports or exports.*, Use ESM export syntax, instead: "+t.id)}}),t),i.o=(t,e)=>Object.prototype.hasOwnProperty.call(t,e),i.r=t=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(t,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(t,"__esModule",{value:!0})},i.j=404,(()=>{var t={404:0};i.O.j=e=>0===t[e];var e=(e,n)=>{var o,r,[a,u,l]=n,s=0;for(o in u)i.o(u,o)&&(i.m[o]=u[o]);if(l)var d=l(i);for(e&&e(n);s<a.length;s++)r=a[s],i.o(t,r)&&t[r]&&t[r][0](),t[a[s]]=0;return i.O(d)},n=globalThis.webpackChunkwagtail=globalThis.webpackChunkwagtail||[];n.forEach(e.bind(null,0)),n.push=e.bind(null,n.push.bind(n))})(),i.O(void 0,[876],(()=>i(40748)));var o=i.O(void 0,[876],(()=>i(90971)));o=i.O(o)})();
//# sourceMappingURL=wagtailadmin.js.map