{% load wagtailadmin %}

<li class="menu-item{% if active %} menu-active{% endif %}">
    <a href="#" data-nav-primary-submenu-trigger class="{{ classnames }}"{{ attr_string }}>
        {% if icon_name %}{% icon icon_name 'icon--menuitem' %}{% endif %}
        {{ label }}
        {% icon 'arrow-right' 'icon--submenu-trigger' %}
    </a>
    <div class="nav-submenu">
        <h2 id="nav-submenu-{{ name }}-title" class="{{ classnames }}">
            {% if icon_name %}{% icon icon_name 'icon--submenu-header' %}{% endif %}
            {{ label }}
        </h2>
        <ul class="nav-submenu__list" aria-labelledby="nav-submenu-{{ name }}-title">
            {{ menu_html }}
        </ul>
        {% block menu_footer %}{% endblock %}
    </div>
</li>
