(()=>{"use strict";var e,t={27578:function(e,t,o){var a=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};t.__esModule=!0;var l=a(o(73609));function r(e){e.tooltip({animation:!1,title:function(){return l.default(this).attr("href")},trigger:"hover",placement:"bottom",selector:"a"})}function n(e){var t=l.default('<a class="icon icon-cross text-replace halloembed__delete">Delete</a>');l.default(e).addClass("halloembed").prepend(t),t.on("click",(function(){var t=l.default(e).parent("[data-hallo-editor]").data("IKS-hallo");l.default(e).fadeOut((function(){l.default(e).remove(),void 0!==t&&t.options.editable&&t.element.trigger("change")}))}))}window.makeHalloRichTextEditable=function(e,t){var o=l.default("#"+e),a=l.default('<div class="halloeditor" data-hallo-editor></div>').html(o.val());a.insertBefore(o),o.hide();var n=!1;function i(){l.default("span[style]",a).filter((function(){return 1===this.attributes.length})).removeAttr("style"),n=!1}var d=o.closest(".object");a.hallo({toolbar:"halloToolbarFixed",toolbarCssClass:d.hasClass("full")?"full":"",plugins:t}).on("hallomodified",(function(e,t){o.val(t.content),n||(setTimeout(i,100),n=!0)})).on("paste drop",(function(){setTimeout((function(){var e;i(),(e=a.data("IKS-hallo"))&&e.setModified()}),1)})).on("halloactivated",(function(e){l.default(e.target).addClass("expanded",200,(function(){l.default(window).trigger("scroll")}))})).on("hallodeactivated",(function(e){l.default(e.target).removeClass("expanded",200,(function(){l.default(window).trigger("scroll")}))})),r(a)},window.setupLinkTooltips=r,window.insertRichTextDeleteControl=n,l.default((function(){l.default('[data-hallo-editor] [contenteditable="false"]').each((function(){n(this)}))}))},73609:e=>{e.exports=jQuery}},o={};function a(e){var l=o[e];if(void 0!==l)return l.exports;var r=o[e]={id:e,loaded:!1,exports:{}};return t[e].call(r.exports,r,r.exports,a),r.loaded=!0,r.exports}a.m=t,e=[],a.O=(t,o,l,r)=>{if(!o){var n=1/0;for(u=0;u<e.length;u++){for(var[o,l,r]=e[u],i=!0,d=0;d<o.length;d++)(!1&r||n>=r)&&Object.keys(a.O).every((e=>a.O[e](o[d])))?o.splice(d--,1):(i=!1,r<n&&(n=r));i&&(e.splice(u--,1),t=l())}return t}r=r||0;for(var u=e.length;u>0&&e[u-1][2]>r;u--)e[u]=e[u-1];e[u]=[o,l,r]},a.n=e=>{var t=e&&e.__esModule?()=>e.default:()=>e;return a.d(t,{a:t}),t},a.d=(e,t)=>{for(var o in t)a.o(t,o)&&!a.o(e,o)&&Object.defineProperty(e,o,{enumerable:!0,get:t[o]})},a.g=function(){if("object"==typeof globalThis)return globalThis;try{return this||new Function("return this")()}catch(e){if("object"==typeof window)return window}}(),a.hmd=e=>((e=Object.create(e)).children||(e.children=[]),Object.defineProperty(e,"exports",{enumerable:!0,set:()=>{throw new Error("ES Modules may not assign module.exports or exports.*, Use ESM export syntax, instead: "+e.id)}}),e),a.o=(e,t)=>Object.prototype.hasOwnProperty.call(e,t),a.r=e=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},a.j=172,(()=>{var e={172:0};a.O.j=t=>0===e[t];var t=(t,o)=>{var l,r,[n,i,d]=o,u=0;for(l in i)a.o(i,l)&&(a.m[l]=i[l]);if(d)var s=d(a);for(t&&t(o);u<n.length;u++)r=n[u],a.o(e,r)&&e[r]&&e[r][0](),e[n[u]]=0;return a.O(s)},o=globalThis.webpackChunkwagtail=globalThis.webpackChunkwagtail||[];o.forEach(t.bind(null,0)),o.push=t.bind(null,o.push.bind(o))})(),a.O(void 0,[876],(()=>a(27578)));var l=a.O(void 0,[876],(()=>a(90971)));l=a.O(l)})();
//# sourceMappingURL=hallo-bootstrap.js.map