# Generated by Django 3.1.8 on 2021-10-01 14:35

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailusers', '0011_userprofile_related_name_change'),
        ('wagtailcore', '0068_copy_userprofiles'),
    ]

    operations = [
        migrations.DeleteModel(
            name='UserProfile',
        ),
    ]
