# Generated by Django 3.1.8 on 2021-06-22 15:36

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailsearchpromotions', '0004_copy_queries'),
    ]

    operations = [
        migrations.AlterField(
            model_name='searchpromotion',
            name='query',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='editors_picks', to='wagtailsearchpromotions.query'),
        ),
    ]
