.typed-table-block__wrapper{overflow-x:auto;min-height:20rem}.typed-table-block table{margin:2.5rem 0 1rem 2.5rem}.typed-table-block th{position:relative}.typed-table-block th button.append-column,.typed-table-block th button.prepend-column{position:absolute;left:-.9rem;top:-2rem}.typed-table-block th button.delete-column{position:absolute;right:.625rem;top:.875rem}.typed-table-block th input.column-heading{padding-right:3rem}.typed-table-block tbody tr,.typed-table-block td,.typed-table-block tfoot tr{position:relative}.typed-table-block tbody tr button.prepend-row,.typed-table-block tfoot tr button.prepend-row{left:-2rem;position:absolute;top:-.9rem}.typed-table-block tbody tr button.delete-row,.typed-table-block tfoot tr button.delete-row{margin-left:.25rem}.typed-table-block tbody tr td:not(.control-cell),.typed-table-block tfoot tr td:not(.control-cell){vertical-align:top;min-width:20rem}.typed-table-block ul.add-column-menu{position:absolute;top:0;left:-.9rem;z-index:100;min-width:10rem;background-color:#fff}.typed-table-block ul.add-column-menu button{width:100%;margin:1px}.typed-table-block .control-cell ul.add-column-menu{top:0}