{% extends 'wagtailadmin/bulk_actions/confirmation/base.html' %}
{% load wagtailadmin i18n %}
{% block titletag %}{% blocktrans count counter=items|length %}Delete 1 image{% plural %}Delete {{ counter }} images{% endblocktrans %}{% endblock %}

{% block header %}
    {% trans "Delete images" as del_str %}
    {% include "wagtailadmin/shared/header.html" with title=del_str icon="doc-full-inverse" %}
{% endblock header %}

{% block items_with_access %}
{% if items %}
    <p>
        {% blocktrans trimmed count counter=items|length %}
        Are you sure you want to delete this image?
        {% plural %}
        Are you sure you want to delete these images?
        {% endblocktrans %}
    </p>
    <ul>
    {% usage_count_enabled as uc_enabled %}
    {% for image in items %}
        <li>
            <a href="{% url 'wagtailimages:edit' image.item.id %}"  target="_blank" rel="noopener noreferrer">{{image.item.title}}</a>
            {% if uc_enabled %}
            (<a href="{{ image.item.usage_url }}">{% blocktrans count usage_count=image.item.get_usage.count %}Used {{ usage_count }} time{% plural %}Used {{ usage_count }} times{% endblocktrans %}</a>)
            {% endif %}
        </li>
    {% endfor %}
    </ul>
{% endif %}
{% endblock items_with_access %}

{% block items_with_no_access %}

{% blocktrans asvar no_access_msg count counter=items_with_no_access|length %}You don't have permission to delete this image{% plural %}You don't have permission to delete these images{% endblocktrans %}
{% include 'wagtailimages/bulk_actions/list_items_with_no_access.html' with items=items_with_no_access no_access_msg=no_access_msg %}

{% endblock items_with_no_access %}

{% block form_section %}
    {% if items %}
        {% trans 'Yes, delete' as action_button_text %}
        {% trans "No, don't delete" as no_action_button_text %}
        {% include 'wagtailadmin/bulk_actions/confirmation/form.html' with action_button_class="serious" %}
    {% else %}
        {% include 'wagtailadmin/bulk_actions/confirmation/go_back.html' %}
    {% endif %}
{% endblock form_section %}
