# Generated by Django 3.1.8 on 2021-10-01 12:44

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('wagtailusers', '0010_userprofile_updated_comments_notifications'),
    ]

    # This migration is a dependency of '0067_copy_userprofiles',
    # but we don't want to add this as a hard-dependency in that
    # migration in case wagtailusers is not installed.
    run_before = [
        ('wagtailcore', '0068_copy_userprofiles'),
    ]

    operations = [
        migrations.AlterField(
            model_name='userprofile',
            name='user',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='wagtail_userprofile_legacy', to=settings.AUTH_USER_MODEL),
        ),
    ]
