# Generated by Django 3.1.8 on 2021-06-22 14:56

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailsearchpromotions', '0003_query_querydailyhits'),
    ]

    operations = [
        migrations.RunSQL('INSERT INTO wagtailsearchpromotions_query SELECT * FROM wagtailsearch_query', ''),
        migrations.RunSQL('INSERT INTO wagtailsearchpromotions_querydailyhits SELECT * FROM wagtailsearch_querydailyhits', ''),
    ]
