# Generated by Django 3.2.3 on 2021-05-28 21:21

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0059_deadlystreampage'),
    ]

    operations = [
        migrations.CreateModel(
            name='TaggedChildPage',
            fields=[
                ('taggedpage_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='tests.taggedpage')),
            ],
            options={
                'abstract': False,
            },
            bases=('tests.taggedpage',),
        ),
        migrations.CreateModel(
            name='TaggedGrandchildPage',
            fields=[
                ('taggedchildpage_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='tests.taggedchildpage')),
            ],
            options={
                'abstract': False,
            },
            bases=('tests.taggedchildpage',),
        ),
    ]
