# Generated by Django 3.0.6 on 2020-05-26 09:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0050_customimagewithauthor_customrenditionwithauthor'),
    ]

    operations = [
        migrations.AlterField(
            model_name='restauranttag',
            name='name',
            field=models.CharField(max_length=100, unique=True, verbose_name='name'),
        ),
        migrations.AlterField(
            model_name='restauranttag',
            name='slug',
            field=models.SlugField(max_length=100, unique=True, verbose_name='slug'),
        ),
    ]
