{% load wagtailadmin %}

{% if start_page_id %}
    <li class="menu-item{% if active %} menu-active{% endif %}" data-explorer-menu-item>
        <a href="{{ url }}" data-explorer-start-page="{{ start_page_id }}">
          {% if icon_name %}{% icon icon_name 'icon--menuitem' %}{% endif %}
          {{ label }}
        </a>
    </li>
{% endif %}
