{% extends "wagtailadmin/shared/header_with_locale_selector.html" %}
{% load wagtailadmin i18n %}

{% block extra_rows %}
<div class="row last-updated">
    {% if latest_log_entry %}
    <ul>
        <li>
            <span class="avatar small" data-wagtail-tooltip="{{ latest_log_entry.user_display_name }}"><img src="{% avatar_url latest_log_entry.user size=25 %}" alt="" /></span>
            {% trans "Last updated" %}
            {% include "wagtailadmin/shared/last_updated.html" with last_updated=latest_log_entry.timestamp time_prefix="at" %}
        </li>
        {% if history_url %}
            <li><a href="{{ history_url }}">{% icon "history" class_name="default" %} {% trans "History" %}</a></li>
        {% endif %}
    </ul>
    {% endif %}
</div>
{% endblock %}
