.top-padding{padding-top:52px}.avatar-panel{margin-top:20px}.avatar-panel__image{float:left;width:16%;margin-bottom:20px}.avatar-panel__form{float:left}.avatar-panel__form label{display:block;padding:0;width:unset;float:unset;margin-bottom:10px}.avatar-panel__revert-to-gravatar{margin-top:20px;margin-bottom:20px}.avatar-panel:after{content:" ";display:table;clear:both}