"""
Temporarily made this into a folder to make it easier to automate the merging
Will concatenate these files later
"""

from .admin import *  # noqa
from .core import *  # noqa
from .locales import *  # noqa
from .sites import *  # noqa
from .users import *  # noqa
from .utils import *  # noqa
