(()=>{var e,r={98274:()=>{var e=function(){function e(e,r){this.html=e,this.idPattern=r}return e.prototype.render=function(e,r,t,o){var n=this.html.replace(/__NAME__/g,r).replace(/__ID__/g,t);e.outerHTML=n;var i=createSnippetChooser(t);return i.setState(o),i},e}();window.telepath.register("wagtail.snippets.widgets.SnippetChooser",e)}},t={};function o(e){var n=t[e];if(void 0!==n)return n.exports;var i=t[e]={id:e,loaded:!1,exports:{}};return r[e].call(i.exports,i,i.exports,o),i.loaded=!0,i.exports}o.m=r,e=[],o.O=(r,t,n,i)=>{if(!t){var a=1/0;for(d=0;d<e.length;d++){for(var[t,n,i]=e[d],l=!0,s=0;s<t.length;s++)(!1&i||a>=i)&&Object.keys(o.O).every((e=>o.O[e](t[s])))?t.splice(s--,1):(l=!1,i<a&&(a=i));l&&(e.splice(d--,1),r=n())}return r}i=i||0;for(var d=e.length;d>0&&e[d-1][2]>i;d--)e[d]=e[d-1];e[d]=[t,n,i]},o.n=e=>{var r=e&&e.__esModule?()=>e.default:()=>e;return o.d(r,{a:r}),r},o.d=(e,r)=>{for(var t in r)o.o(r,t)&&!o.o(e,t)&&Object.defineProperty(e,t,{enumerable:!0,get:r[t]})},o.g=function(){if("object"==typeof globalThis)return globalThis;try{return this||new Function("return this")()}catch(e){if("object"==typeof window)return window}}(),o.hmd=e=>((e=Object.create(e)).children||(e.children=[]),Object.defineProperty(e,"exports",{enumerable:!0,set:()=>{throw new Error("ES Modules may not assign module.exports or exports.*, Use ESM export syntax, instead: "+e.id)}}),e),o.o=(e,r)=>Object.prototype.hasOwnProperty.call(e,r),o.r=e=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},o.j=383,(()=>{var e={383:0};o.O.j=r=>0===e[r];var r=(r,t)=>{var n,i,[a,l,s]=t,d=0;for(n in l)o.o(l,n)&&(o.m[n]=l[n]);if(s)var p=s(o);for(r&&r(t);d<a.length;d++)i=a[d],o.o(e,i)&&e[i]&&e[i][0](),e[a[d]]=0;return o.O(p)},t=globalThis.webpackChunkwagtail=globalThis.webpackChunkwagtail||[];t.forEach(r.bind(null,0)),t.push=r.bind(null,t.push.bind(t))})(),o.O(void 0,[876],(()=>o(98274)));var n=o.O(void 0,[876],(()=>o(90971)));n=o.O(n)})();
//# sourceMappingURL=snippet-chooser-telepath.js.map