# Generated by Django 3.2.5 on 2021-07-20 12:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0063_modellogentry'),
    ]

    operations = [
        migrations.AlterField(
            model_name='modellogentry',
            name='timestamp',
            field=models.DateTimeField(db_index=True, verbose_name='timestamp (UTC)'),
        ),
        migrations.AlterField(
            model_name='pagelogentry',
            name='timestamp',
            field=models.DateTimeField(db_index=True, verbose_name='timestamp (UTC)'),
        ),
    ]
