# Generated by Django 3.1 on 2020-08-13 21:28

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('customuser', '0002_added_file_field'),
    ]

    operations = [
        migrations.DeleteModel(
            name='EmailUser',
        ),
    ]
