{% load wagtailadmin %}

<li class="menu-item{% if active %} menu-active{% endif %}">
    <a href="{{ url }}"
       class="{{ classnames }}"
       {{ attr_string }}>
        {% if icon_name %}{% icon icon_name 'icon--menuitem' %}{% endif %}
        {{ label }}
    </a>
</li>
