(()=>{"use strict";var e,r={923:function(e,r,o){var t=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};r.__esModule=!0;var n=t(o(73609));window.createTaskChooser=function(e){var r=n.default("#"+e+"-chooser"),o=r.find(".name"),t=n.default("#"+e),a=r.find(".action-edit");n.default(".action-choose",r).on("click",(function(){ModalWorkflow({url:r.data("chooserUrl"),onload:TASK_CHOOSER_MODAL_ONLOAD_HANDLERS,responses:{taskChosen:function(e){t.val(e.id),o.text(e.name),r.removeClass("blank"),a.attr("href",e.edit_url)}}})}))}},73609:e=>{e.exports=jQuery}},o={};function t(e){var n=o[e];if(void 0!==n)return n.exports;var a=o[e]={id:e,loaded:!1,exports:{}};return r[e].call(a.exports,a,a.exports,t),a.loaded=!0,a.exports}t.m=r,e=[],t.O=(r,o,n,a)=>{if(!o){var i=1/0;for(d=0;d<e.length;d++){for(var[o,n,a]=e[d],l=!0,s=0;s<o.length;s++)(!1&a||i>=a)&&Object.keys(t.O).every((e=>t.O[e](o[s])))?o.splice(s--,1):(l=!1,a<i&&(i=a));l&&(e.splice(d--,1),r=n())}return r}a=a||0;for(var d=e.length;d>0&&e[d-1][2]>a;d--)e[d]=e[d-1];e[d]=[o,n,a]},t.n=e=>{var r=e&&e.__esModule?()=>e.default:()=>e;return t.d(r,{a:r}),r},t.d=(e,r)=>{for(var o in r)t.o(r,o)&&!t.o(e,o)&&Object.defineProperty(e,o,{enumerable:!0,get:r[o]})},t.g=function(){if("object"==typeof globalThis)return globalThis;try{return this||new Function("return this")()}catch(e){if("object"==typeof window)return window}}(),t.hmd=e=>((e=Object.create(e)).children||(e.children=[]),Object.defineProperty(e,"exports",{enumerable:!0,set:()=>{throw new Error("ES Modules may not assign module.exports or exports.*, Use ESM export syntax, instead: "+e.id)}}),e),t.o=(e,r)=>Object.prototype.hasOwnProperty.call(e,r),t.r=e=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},t.j=343,(()=>{var e={343:0};t.O.j=r=>0===e[r];var r=(r,o)=>{var n,a,[i,l,s]=o,d=0;for(n in l)t.o(l,n)&&(t.m[n]=l[n]);if(s)var u=s(t);for(r&&r(o);d<i.length;d++)a=i[d],t.o(e,a)&&e[a]&&e[a][0](),e[i[d]]=0;return t.O(u)},o=globalThis.webpackChunkwagtail=globalThis.webpackChunkwagtail||[];o.forEach(r.bind(null,0)),o.push=r.bind(null,o.push.bind(o))})(),t.O(void 0,[876],(()=>t(923)));var n=t.O(void 0,[876],(()=>t(90971)));n=t.O(n)})();
//# sourceMappingURL=task-chooser.js.map