(()=>{"use strict";var e,t={34972:function(e,t,r){var o=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};t.__esModule=!0;var n=o(r(73609));n.default.widget("IKS.hallorequireparagraphs",{options:{editable:null,uuid:"",blockElements:["dd","div","dl","figure","form","ul","ol","table","p","h1","h2","h3","h4","h5","h6"]},cleanupContentClone:function(e){e.html().length&&!n.default(this.options.blockElements.toString(),e).length&&this.options.editable.execute("formatBlock","p")}})},73609:e=>{e.exports=jQuery}},r={};function o(e){var n=r[e];if(void 0!==n)return n.exports;var l=r[e]={id:e,loaded:!1,exports:{}};return t[e].call(l.exports,l,l.exports,o),l.loaded=!0,l.exports}o.m=t,e=[],o.O=(t,r,n,l)=>{if(!r){var i=1/0;for(d=0;d<e.length;d++){for(var[r,n,l]=e[d],a=!0,u=0;u<r.length;u++)(!1&l||i>=l)&&Object.keys(o.O).every((e=>o.O[e](r[u])))?r.splice(u--,1):(a=!1,l<i&&(i=l));a&&(e.splice(d--,1),t=n())}return t}l=l||0;for(var d=e.length;d>0&&e[d-1][2]>l;d--)e[d]=e[d-1];e[d]=[r,n,l]},o.n=e=>{var t=e&&e.__esModule?()=>e.default:()=>e;return o.d(t,{a:t}),t},o.d=(e,t)=>{for(var r in t)o.o(t,r)&&!o.o(e,r)&&Object.defineProperty(e,r,{enumerable:!0,get:t[r]})},o.g=function(){if("object"==typeof globalThis)return globalThis;try{return this||new Function("return this")()}catch(e){if("object"==typeof window)return window}}(),o.hmd=e=>((e=Object.create(e)).children||(e.children=[]),Object.defineProperty(e,"exports",{enumerable:!0,set:()=>{throw new Error("ES Modules may not assign module.exports or exports.*, Use ESM export syntax, instead: "+e.id)}}),e),o.o=(e,t)=>Object.prototype.hasOwnProperty.call(e,t),o.r=e=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},o.j=61,(()=>{var e={61:0};o.O.j=t=>0===e[t];var t=(t,r)=>{var n,l,[i,a,u]=r,d=0;for(n in a)o.o(a,n)&&(o.m[n]=a[n]);if(u)var s=u(o);for(t&&t(r);d<i.length;d++)l=i[d],o.o(e,l)&&e[l]&&e[l][0](),e[i[d]]=0;return o.O(s)},r=globalThis.webpackChunkwagtail=globalThis.webpackChunkwagtail||[];r.forEach(t.bind(null,0)),r.push=t.bind(null,r.push.bind(r))})(),o.O(void 0,[876],(()=>o(34972)));var n=o.O(void 0,[876],(()=>o(90971)));n=o.O(n)})();
//# sourceMappingURL=hallo-requireparagraphs.js.map