# Generated by Django 3.2.8 on 2021-10-28 18:17

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailadmin', '0003_admin_managed'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Admin',
        ),
    ]
