(()=>{"use strict";var e,t={88002:function(e,t,n){var a=this&&this.__importDefault||function(e){return e&&e.__esModule?e:{default:e}};t.__esModule=!0;var o=a(n(73609));function i(e,t){return e.getDate()===t.getDate()&&e.getMonth()===t.getMonth()&&e.getYear()===t.getYear()}function r(e,t){i(new Date(t[0].value),e)||o.default(this).find(".xdsoft_datepicker .xdsoft_current:not(.xdsoft_today)").removeClass("xdsoft_current")}o.default.fn.datetimepicker.defaults.i18n.wagtail_custom_locale={months:wagtailConfig.STRINGS.MONTHS,dayOfWeek:wagtailConfig.STRINGS.WEEKDAYS,dayOfWeekShort:wagtailConfig.STRINGS.WEEKDAYS_SHORT},o.default.datetimepicker.setLocale("wagtail_custom_locale"),window.dateEqual=i,window.hideCurrent=r,window.initDateChooser=function(e,t){window.dateTimePickerTranslations?o.default("#"+e).datetimepicker(o.default.extend({closeOnDateSelect:!0,timepicker:!1,scrollInput:!1,format:"Y-m-d",onGenerate:r,onChangeDateTime:function(e,t){t.get(0).dispatchEvent(new Event("change"))}},t||{})):o.default("#"+e).datetimepicker(o.default.extend({timepicker:!1,scrollInput:!1,format:"Y-m-d",onGenerate:r,onChangeDateTime:function(e,t){t.get(0).dispatchEvent(new Event("change"))}},t||{}))},window.initTimeChooser=function(e,t){window.dateTimePickerTranslations?o.default("#"+e).datetimepicker(o.default.extend({closeOnDateSelect:!0,datepicker:!1,scrollInput:!1,format:"H:i",onChangeDateTime:function(e,t){t.get(0).dispatchEvent(new Event("change"))}},t||{})):o.default("#"+e).datetimepicker(o.default.extend({datepicker:!1,format:"H:i",onChangeDateTime:function(e,t){t.get(0).dispatchEvent(new Event("change"))}},t||{}))},window.initDateTimeChooser=function(e,t){window.dateTimePickerTranslations?o.default("#"+e).datetimepicker(o.default.extend({closeOnDateSelect:!0,format:"Y-m-d H:i",scrollInput:!1,onGenerate:r,onChangeDateTime:function(e,t){t.get(0).dispatchEvent(new Event("change"))}},t||{})):o.default("#"+e).datetimepicker(o.default.extend({format:"Y-m-d H:i",onGenerate:r,onChangeDateTime:function(e,t){t.get(0).dispatchEvent(new Event("change"))}},t||{}))}},73609:e=>{e.exports=jQuery}},n={};function a(e){var o=n[e];if(void 0!==o)return o.exports;var i=n[e]={id:e,loaded:!1,exports:{}};return t[e].call(i.exports,i,i.exports,a),i.loaded=!0,i.exports}a.m=t,e=[],a.O=(t,n,o,i)=>{if(!n){var r=1/0;for(c=0;c<e.length;c++){for(var[n,o,i]=e[c],d=!0,l=0;l<n.length;l++)(!1&i||r>=i)&&Object.keys(a.O).every((e=>a.O[e](n[l])))?n.splice(l--,1):(d=!1,i<r&&(r=i));d&&(e.splice(c--,1),t=o())}return t}i=i||0;for(var c=e.length;c>0&&e[c-1][2]>i;c--)e[c]=e[c-1];e[c]=[n,o,i]},a.n=e=>{var t=e&&e.__esModule?()=>e.default:()=>e;return a.d(t,{a:t}),t},a.d=(e,t)=>{for(var n in t)a.o(t,n)&&!a.o(e,n)&&Object.defineProperty(e,n,{enumerable:!0,get:t[n]})},a.g=function(){if("object"==typeof globalThis)return globalThis;try{return this||new Function("return this")()}catch(e){if("object"==typeof window)return window}}(),a.hmd=e=>((e=Object.create(e)).children||(e.children=[]),Object.defineProperty(e,"exports",{enumerable:!0,set:()=>{throw new Error("ES Modules may not assign module.exports or exports.*, Use ESM export syntax, instead: "+e.id)}}),e),a.o=(e,t)=>Object.prototype.hasOwnProperty.call(e,t),a.r=e=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},a.j=465,(()=>{var e={465:0};a.O.j=t=>0===e[t];var t=(t,n)=>{var o,i,[r,d,l]=n,c=0;for(o in d)a.o(d,o)&&(a.m[o]=d[o]);if(l)var u=l(a);for(t&&t(n);c<r.length;c++)i=r[c],a.o(e,i)&&e[i]&&e[i][0](),e[r[c]]=0;return a.O(u)},n=globalThis.webpackChunkwagtail=globalThis.webpackChunkwagtail||[];n.forEach(t.bind(null,0)),n.push=t.bind(null,n.push.bind(n))})(),a.O(void 0,[876],(()=>a(88002)));var o=a.O(void 0,[876],(()=>a(90971)));o=a.O(o)})();
//# sourceMappingURL=date-time-chooser.js.map