# Generated by Django 2.2.10 on 2020-07-13 10:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0052_pagelogentry'),
    ]

    operations = [
        migrations.CreateModel(
            name='Locale',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(max_length=100, unique=True)),
            ],
            options={
                'ordering': ['language_code'],
            },
        ),
    ]
