# Generated by Django 2.2.16 on 2021-04-12 20:22

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='SimpleTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'permissions': [('submit_translation', 'Can submit translations')],
                'default_permissions': [],
            },
        ),
    ]
