{% extends "wagtailadmin/userbar/item_base.html" %}
{% load i18n wagtailadmin %}

{% block item_content %}
    <a href="{% url 'wagtailadmin_pages:add_subpage' self.page.id %}" target="_parent" role="menuitem">
      {% icon name="plus" class_name="wagtail-action-icon" %}
      {% trans 'Add a child page' %}
    </a>
{% endblock %}
