# Generated by Django 2.2.10 on 2020-07-13 10:13

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0054_initial_locale'),
    ]

    operations = [
        migrations.AddField(
            model_name='page',
            name='translation_key',
            field=models.UUIDField(editable=False, null=True),
        ),
        migrations.AddField(
            model_name='page',
            name='locale',
            field=models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='wagtailcore.Locale'),
        ),
        migrations.AlterUniqueTogether(
            name='page',
            unique_together={('translation_key', 'locale')},
        ),
    ]
